package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.ABehospitalInfoDTO;
import com.diagbot.entity.*;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.BehospitalInfoServiceImpl;
import com.diagbot.service.impl.MedicalRecordServiceImpl;
import com.diagbot.service.impl.QcAbnormalServiceImpl;
import com.diagbot.service.impl.QcTypeServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.TZDBConn;
import com.diagbot.vo.data.ABehospitalInfoVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

@Component
public class ABehospitalInfoFacade {
    @Autowired
    @Qualifier("behospitalInfoServiceImpl")
    private BehospitalInfoServiceImpl behospitalInfoServiceImpl;

    @Autowired
    private QcTypeFacade qcTypeFacade;

    @Autowired
    @Qualifier("medicalRecordServiceImpl")
    private MedicalRecordServiceImpl medicalRecordServiceImpl;

    @Autowired
    private QcAbnormalFacade qcAbnormalFacade;

    private TZDBConn tzDBConn = new TZDBConn();

    /**
     * 终末质控-同步前一天的入院记录
     */
    public void executeBehospital() {
        //String sql="select * from br_inpatientinfo where cjcxrq>=dateadd(day,-2,getdate()) and cjcxrq<=getdate()";
        String sql="select * from br_inpatientinfo where cjcxrq>=(select CONVERT(varchar,GETDATE()-1,23)) and cjcxrq<(select CONVERT(varchar,GETDATE(),23)) ORDER BY cjcxrq DESC";
        List<BehospitalInfo> behospitalInfoList = tzDBConn.getBehospitalInfo(sql);
        execute(behospitalInfoList);
    }

    /**
     * 同步前一天的入院记录
     */
    public void executeBehospitalPast() {
        String sql="select * from br_inpatientinfo where cjcxrq>=dateadd(day,-2,getdate()) and cjcxrq<=getdate()";
        List<BehospitalInfo> behospitalInfoList = tzDBConn.getBehospitalInfo(sql);
        execute(behospitalInfoList);
    }

    /**
     * 通过接口更新病人住院记录信息
     * @param list
     * @return
     */
    public RespDTO<List<ABehospitalInfoDTO>> executeBehospital(List<ABehospitalInfoVO> list) {
        if(list!=null && list.size()>0) {
            List<BehospitalInfo> behospitalInfoList = Lists.newArrayList();
            List<ABehospitalInfoDTO> behospitalInfoDtoList = Lists.newArrayList();
            behospitalInfoList = BeanUtil.listCopyTo(list, BehospitalInfo.class);
            behospitalInfoDtoList = BeanUtil.listCopyTo(list, ABehospitalInfoDTO.class);

            //循环验证数据有效性
            for (BehospitalInfo behospitalInfo:behospitalInfoList) {
                if("".equals(behospitalInfo.getBehospitalCode())) {
                    return RespDTO.onError("请输入病人住院编码!");
                }else if(behospitalInfo.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }else if("".equals(behospitalInfo.getBehDeptId())){
                    return RespDTO.onError("请输入住院科室编码!");
                }else if("".equals(behospitalInfo.getBehDeptName())){
                    return RespDTO.onError("请输入住院科室名称!");
                }
            }
            execute(behospitalInfoList);
            return RespDTO.onSuc(behospitalInfoDtoList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void executeBehospitalPort(){

    }

    public void execute(List<BehospitalInfo> behospitalInfoList){
        List<BehospitalInfo> add = Lists.newLinkedList();
        List<BehospitalInfo> updateE = Lists.newLinkedList();
        behospitalInfoList.stream().forEach(s -> {
            QueryWrapper<MedicalRecord> wrapper = new QueryWrapper<>();
            wrapper.eq("behospital_code", s.getBehospitalCode());
            wrapper.eq("hospital_id", s.getHospitalId());
            wrapper.eq("mode_id", Long.valueOf("1"));
            MedicalRecord medicalRecord = medicalRecordServiceImpl.getOne(wrapper);
            if(medicalRecord!=null){
                QueryWrapper<QcType> qWrapper =new QueryWrapper<>();
                qWrapper.eq("name",medicalRecord.getRecTitle());
                qWrapper.eq("hospital_id", s.getHospitalId());
                QcType qcType=qcTypeFacade.getOne(qWrapper);
                if(qcType!=null){
                    s.setQcTypeId(qcType.getId());
                }else{
                    //新增类型到qc_type表中
                    QcType qctype=new QcType();
                    qctype.setHospitalId(s.getHospitalId());
                    qctype.setName(medicalRecord.getRecTitle());
                    qctype.setRemark("台州");
                    qctype.setGmtCreate(new Date());

                    Long id=qcTypeFacade.saveT(qctype);
                    s.setQcTypeId(id);

                    //新增类型到监测表中
                    QcAbnormal qcAbnormal=new QcAbnormal();
                    qcAbnormal.setHospitalId(s.getHospitalId());
                    qcAbnormal.setBehospitalCode(s.getBehospitalCode());
                    qcAbnormal.setType(2);
                    qcAbnormal.setDescription(medicalRecord.getRecTitle());
                    qcAbnormal.setGmtCreate(new Date());
                    qcAbnormalFacade.getBaseMapper().insert(qcAbnormal);
                }
            }

            QueryWrapper<BehospitalInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("behospital_code", s.getBehospitalCode());
            queryWrapper.eq("hospital_id", s.getHospitalId());
            queryWrapper.eq("is_deleted",IsDeleteEnum.N);
            BehospitalInfo behospitalInfo = behospitalInfoServiceImpl.getOne(queryWrapper);
            if (behospitalInfo != null) {
                s.setGmtModified(new Date());
                behospitalInfoServiceImpl.update(s,queryWrapper);
            } else {
                s.setGmtCreate(new Date());
                behospitalInfoServiceImpl.save(s);
            }
        });
    }
}
