package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.ADoctorAdviceDTO;
import com.diagbot.entity.BehospitalInfo;
import com.diagbot.entity.DoctorAdvice;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.DoctorAdviceServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.TZDBConn;
import com.diagbot.vo.data.ADoctorAdviceVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

@Component
public class ADoctorAdviceFacade {
    @Autowired
    @Qualifier("doctorAdviceServiceImpl")
    private DoctorAdviceServiceImpl doctorAdviceServiceImpl;

    private TZDBConn tzDBConn = new TZDBConn();

    /**
     * 终末质控-同步前一天的入院病人医嘱信息
     */
    public void executeDoctorAdvice() {
        //String sql="select * from br_doctadvice where cjcxrq>=dateadd(day,-2,getdate()) and cjcxrq<=getdate()";
        String sql="select * from br_doctadvice where cjcxrq>=(select CONVERT(varchar,GETDATE()-1,23)) and cjcxrq<(select CONVERT(varchar,GETDATE(),23)) ORDER BY cjcxrq DESC";
        List<DoctorAdvice> doctorAdviceList = tzDBConn.getDoctorAdvice(sql);
        execute(doctorAdviceList);
    }

    public void executeDoctorAdvicePast() {
        String sql="select * from br_doctadvice where cjcxrq>=dateadd(day,-2,getdate()) and cjcxrq<=getdate()";
        List<DoctorAdvice> doctorAdviceList = tzDBConn.getDoctorAdvice(sql);
        execute(doctorAdviceList);
    }

    public void executeDoctorAdvicePort(){

    }

    /**
     * 数据引擎-通过接口更新医生医嘱信息
     * @param list
     * @return
     */
    public RespDTO<List<ADoctorAdviceDTO>> executeDoctorAdvice(List<ADoctorAdviceVO> list) {
        if(list!=null && list.size()>0){
            List<DoctorAdvice> doctorAdviceList=Lists.newArrayList();
            List<ADoctorAdviceDTO> doctorAdviceDtoList=Lists.newArrayList();
            doctorAdviceList=BeanUtil.listCopyTo(list,DoctorAdvice.class);
            doctorAdviceDtoList=BeanUtil.listCopyTo(list,ADoctorAdviceDTO.class);
            //循环验证数据有效性
            for (DoctorAdvice doctorAdvice:doctorAdviceList) {
                if("".equals(doctorAdvice.getDoctorAdviceId())) {
                    return RespDTO.onError("请输入病人医嘱编码!");
                }else if(doctorAdvice.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }else if("".equals(doctorAdvice.getBehospitalCode())){
                    return RespDTO.onError("请输入病人住院编码!");
                }
            }
            execute(doctorAdviceList);
            return RespDTO.onSuc(doctorAdviceDtoList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void execute(List<DoctorAdvice> doctorAdviceList){
        if (doctorAdviceList != null && doctorAdviceList.size() > 0) {
            doctorAdviceList.stream().forEach(s -> {
                QueryWrapper<DoctorAdvice> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("doctor_advice_id", s.getDoctorAdviceId());
                queryWrapper.eq("hospital_id", s.getHospitalId());
                queryWrapper.eq("is_deleted",IsDeleteEnum.N);
                DoctorAdvice doctorAdvice = doctorAdviceServiceImpl.getOne(queryWrapper);
                if (doctorAdvice != null) {
                    s.setGmtModified(new Date());
                    doctorAdviceServiceImpl.update(s,queryWrapper);
                } else {
                    s.setGmtCreate(new Date());
                    doctorAdviceServiceImpl.save(s);
                }
            });
        }
    }
}
