package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.AHomePageDTO;
import com.diagbot.entity.DoctorAdvice;
import com.diagbot.entity.HomeDiagnoseInfo;
import com.diagbot.entity.HomeOperationInfo;
import com.diagbot.entity.HomePage;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.HomeDiagnoseInfoServiceImpl;
import com.diagbot.service.impl.HomeOperationInfoServiceImpl;
import com.diagbot.service.impl.HomePageServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.TZDBConn;
import com.diagbot.vo.data.AHomeDiagnoseVO;
import com.diagbot.vo.data.AHomeOperationVO;
import com.diagbot.vo.data.AHomePageVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

@Component
public class AHomePageFacade {
    @Autowired
    @Qualifier("homePageServiceImpl")
    private HomePageServiceImpl homePageServiceImpl;

    @Autowired
    @Qualifier("homeDiagnoseInfoServiceImpl")
    private HomeDiagnoseInfoServiceImpl homeDiagnoseInfoServiceImpl;

    @Autowired
    @Qualifier("homeOperationInfoServiceImpl")
    private HomeOperationInfoServiceImpl homeOperationInfoServiceImpl;

    private TZDBConn tzDBConn = new TZDBConn();

    /**
     * 同步前一天病案首页
     */
    public void executeHomePage() {
        //String sql="select * from br_rechome where cjcxrq>=dateadd(day,-2,getdate()) and cjcxrq<=getdate()";
        //同步前一天的数据
        String sql="select * from br_rechome where cjcxrq>=(select CONVERT(varchar,GETDATE()-1,23)) and cjcxrq<(select CONVERT(varchar,GETDATE(),23)) ORDER BY cjcxrq DESC";
        List<HomePage> homePageList = tzDBConn.getHomePage(sql);
        execute(homePageList);
    }

    public void executeHomePagePast() {
        String sql="select * from br_rechome where cjcxrq>=dateadd(day,-2,getdate()) and cjcxrq<=getdate()";
        List<HomePage> homePageList = tzDBConn.getHomePage(sql);
        execute(homePageList);
    }

    public void executeHomePagePort(){

    }

    /**
     * 数据引擎-通过接口更新病案首页
     * @param list
     * @return
     */
    public RespDTO<List<AHomePageDTO>> executeHomePage(List<AHomePageVO> list) {
        if(list!=null && list.size()>0){
            //循环验证数据有效性
            for (AHomePageVO aHomePageVO:list) {
                if("".equals(aHomePageVO.getHomePageId())) {
                    return RespDTO.onError("请输入病案首页编号!");
                }else if(aHomePageVO.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }else if("".equals(aHomePageVO.getBehospitalCode())){
                    return RespDTO.onError("请输入病人住院编码!");
                }
                //出院诊断
                if(aHomePageVO.getLeaveDiags()!=null && aHomePageVO.getLeaveDiags().size()>0){
                    for (AHomeDiagnoseVO aHomeDiagnoseVO:aHomePageVO.getLeaveDiags()){
                        if("".equals(aHomeDiagnoseVO.getHomePageId())) {
                            return RespDTO.onError("出院诊断未输入病案首页编号!");
                        }else if(aHomeDiagnoseVO.getHospitalId()==null){
                            return RespDTO.onError("出院诊断未输入医院编码!");
                        }else if("".equals(aHomeDiagnoseVO.getDiagnoseOrderNo())){
                            return RespDTO.onError("出院诊断未输入顺序号!");
                        }
                    }
                };

                //住院手术
                if(aHomePageVO.getOperations()!=null && aHomePageVO.getOperations().size()>0){
                    for (AHomeOperationVO aHomeOperationVO:aHomePageVO.getOperations()){
                        if("".equals(aHomeOperationVO.getHomePageId())) {
                            return RespDTO.onError("住院手术未输入病案首页编号!");
                        }else if(aHomeOperationVO.getHospitalId()==null){
                            return RespDTO.onError("住院手术未输入医院编码!");
                        }else if("".equals(aHomeOperationVO.getOperationOrderNo())){
                            return RespDTO.onError("住院手术未输入顺序号!");
                        }
                    }
                }
            }
            //装载数据
            List<HomePage> homePageList=Lists.newArrayList();
            List<AHomePageDTO> homePageDtoList=Lists.newArrayList();
            list.stream().forEach(s->{
                //初始化入参
                HomePage homePage=new HomePage();
                BeanUtil.copyProperties(s,homePage);
                homePage.setBirthday(DateUtil.parseDate(s.getBirthday()));
                homePage.setBehospitalDate(DateUtil.parseDateTime(s.getBehospitalDate()));
                homePage.setLeaveHospitalDate(DateUtil.parseDateTime(s.getLeaveHospitalDate()));
                homePage.setQcDate(DateUtil.parseDate(s.getQcDate()));
                homePageList.add(homePage);

                //初始化出参
                AHomePageDTO aHomePageDTO=new AHomePageDTO();
                BeanUtil.copyProperties(s,aHomePageDTO);
                homePageDtoList.add(aHomePageDTO);

                //初始化诊断和手术，并同步到数据库
                List<HomeDiagnoseInfo> homeDiagnoseList=BeanUtil.listCopyTo(s.getLeaveDiags(),HomeDiagnoseInfo.class);
                List<HomeOperationInfo> homeOperationList=BeanUtil.listCopyTo(s.getOperations(),HomeOperationInfo.class);

                executeDiagnose(homeDiagnoseList);
                executeOperation(homeOperationList);
            });
            execute(homePageList);

            return RespDTO.onSuc(homePageDtoList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }

    }

    /**
     * 病案首页
     * @param homePageList
     */
    public void execute(List<HomePage> homePageList){
        if (homePageList != null && homePageList.size() > 0) {
            homePageList.stream().forEach(s -> {
                QueryWrapper<HomePage> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("home_page_id", s.getHomePageId());
                queryWrapper.eq("hospital_id", s.getHospitalId());
                queryWrapper.eq("behospital_code", s.getBehospitalCode());
                queryWrapper.eq("is_deleted",IsDeleteEnum.N);
                HomePage homePage = homePageServiceImpl.getOne(queryWrapper);
                if (homePage != null) {
                    s.setGmtModified(new Date());
                    homePageServiceImpl.update(s,queryWrapper);
                } else {
                    s.setGmtCreate(new Date());
                    homePageServiceImpl.save(s);
                }
            });
        }
    }

    /**
     * 出院诊断
     * @param homeDiagnoseList
     */
    public void executeDiagnose(List<HomeDiagnoseInfo> homeDiagnoseList){
        if (homeDiagnoseList != null && homeDiagnoseList.size() > 0) {
            homeDiagnoseList.stream().forEach(s -> {
                if(s.getHomePageId()!=null && !"".equals(s.getDiagnoseOrderNo())){
                    QueryWrapper<HomeDiagnoseInfo> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("home_page_id", s.getHomePageId());
                    queryWrapper.eq("diagnose_order_no", s.getDiagnoseOrderNo());
                    queryWrapper.eq("hospital_id", s.getHospitalId());
                    queryWrapper.eq("is_deleted",IsDeleteEnum.N);
                    HomeDiagnoseInfo diagnoseInfo = homeDiagnoseInfoServiceImpl.getOne(queryWrapper);
                    if (diagnoseInfo != null) {
                        s.setGmtModified(new Date());
                        homeDiagnoseInfoServiceImpl.update(s,queryWrapper);
                    } else {
                        s.setGmtCreate(new Date());
                        homeDiagnoseInfoServiceImpl.save(s);
                    }
                }
            });
        }
    }

    /**
     * 手术情况
     * @param homeOperationList
     */
    public void executeOperation(List<HomeOperationInfo> homeOperationList){
        if (homeOperationList != null && homeOperationList.size() > 0) {
            homeOperationList.stream().forEach(s -> {
                if(s.getHomePageId()!=null && !"".equals(s.getOperationOrderNo())) {
                    QueryWrapper<HomeOperationInfo> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("home_page_id", s.getHomePageId());
                    queryWrapper.eq("operation_order_no", s.getOperationOrderNo());
                    queryWrapper.eq("hospital_id", s.getHospitalId());
                    queryWrapper.eq("is_deleted",IsDeleteEnum.N);
                    HomeOperationInfo operationInfo = homeOperationInfoServiceImpl.getOne(queryWrapper);
                    if (operationInfo != null) {
                        s.setGmtModified(new Date());
                        homeOperationInfoServiceImpl.update(s,queryWrapper);
                    } else {
                        s.setGmtCreate(new Date());
                        homeOperationInfoServiceImpl.save(s);
                    }
                }
            });
        }
    }
}
