package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.AMedicalRecordTypeDTO;
import com.diagbot.entity.BasDoctorInfo;
import com.diagbot.entity.MedRecordType;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.MedRecordTypeServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.TZDBConn;
import com.diagbot.vo.data.AMedicalRecordTypeVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.List;

@Component
public class AMedRecordTypeFacade {
    @Autowired
    @Qualifier("medRecordTypeServiceImpl")
    private MedRecordTypeServiceImpl medRecordTypeServiceImpl;

    private TZDBConn tzDBConn = new TZDBConn();

    /**
     * 终末质控-同步前一天变更的模板类型
     */
    public void executeMRType() {
        String sql="select * from ar_medirecclass where SFZFPB='1' and cjcxrq>=dateadd(day,-2,getdate()) and cjcxrq<=getdate()";
        List<MedRecordType> medRecordTypeList = tzDBConn.getMedicalRecordType(sql);
        execute(medRecordTypeList);
    }

    /**
     * 同步前一天变更的模板类型
     */
    public void executeMRTypePast() {
        String sql="select * from ar_medirecclass where SFZFPB='1' and cjcxrq>=dateadd(day,-2,getdate()) and cjcxrq<=getdate()";
        List<MedRecordType> medRecordTypeList = tzDBConn.getMedicalRecordType(sql);
        execute(medRecordTypeList);
    }

    /**
     * 数据引擎-通过接口更新病人住院记录信息
     * @param list
     * @return
     */
    public RespDTO<List<AMedicalRecordTypeDTO>> executeMRType(List<AMedicalRecordTypeVO> list) {
        if(list!=null && list.size()>0){
            List<MedRecordType> medRecordTypeList=Lists.newArrayList();
            List<AMedicalRecordTypeDTO> medRecordTypeDtoList=Lists.newArrayList();
            medRecordTypeList=BeanUtil.listCopyTo(list,MedRecordType.class);
            medRecordTypeDtoList=BeanUtil.listCopyTo(list,AMedicalRecordTypeDTO.class);
            //循环验证数据有效性
            for (MedRecordType medRecordType:medRecordTypeList) {
                if("".equals(medRecordType.getTypeId())) {
                    return RespDTO.onError("请输入文书类型编码!");
                }else if(medRecordType.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }else if("".equals(medRecordType.getTypeName())){
                    return RespDTO.onError("请文书类型名称!");
                }
            }
            execute(medRecordTypeList);
            return RespDTO.onSuc(medRecordTypeDtoList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void executeMRTypePort(){

    }

    public void execute(List<MedRecordType> medRecordTypeList){
        List<MedRecordType> add = Lists.newLinkedList();
        List<MedRecordType> updateE = Lists.newLinkedList();
        if (medRecordTypeList != null && medRecordTypeList.size() != 0) {
            medRecordTypeList.stream().forEach(s -> {
                QueryWrapper<MedRecordType> queryWrapper = new QueryWrapper<>();
                queryWrapper.in("type_id", s.getTypeId());
                queryWrapper.eq("hospital_id", s.getHospitalId());
                //queryWrapper.eq("is_deleted",IsDeleteEnum.N);
                MedRecordType mrType = medRecordTypeServiceImpl.getOne(queryWrapper);
                if (mrType != null) {
                    medRecordTypeServiceImpl.update(s,queryWrapper);
                } else {
                    medRecordTypeServiceImpl.save(s);
                }
            });
        }
    }
}
