package com.diagbot.web;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.*;
import com.diagbot.facade.*;
import com.diagbot.vo.data.*;
import io.swagger.annotations.ApiOperation;
import jdk.nashorn.internal.ir.annotations.Ignore;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


import io.swagger.annotations.Api;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/qc/data")
@Api(value = "对接接口API", tags = { "对接接口API" })
public class DataController {

    @Autowired
    private ABasDoctorInfoFacade aBasDoctorInfoFacade;

    @Autowired
    private ABasDeptInfoFacade aBasDeptInfoFacade;

    @Autowired
    private AMedRecordTypeFacade aMedRecordTypeFacade;

    @Autowired
    private ABehospitalInfoFacade aBehospitalInfoFacade;

    @Autowired
    private ADoctorAdviceFacade aDoctorAdviceFacade;

    @Autowired
    private AHomePageFacade aHomePageFacade;

    @Autowired
    private AHomeDiagnoseInfoFacade aHomeDiagnoseInfoFacade;

    @Autowired
    private AHomeOperationInfoFacade aHomeOperationInfoFacade;

    @Autowired
    private AMedicalRecordFacade aMedicalRecordFacade;

    @Autowired
    private AMedicalRecordContentFacade aMedicalRecordContentFacade;

    @Autowired
    private AMedCrisisFacade aMedCrisisFacade;

    @ApiOperation(value = "数据引擎-获取医院所有在职医生的基本信息")
    @PostMapping("/sendDoctorInfos")
    @SysLogger("sendDoctorInfos")
    public RespDTO<List<ABasDoctorInfoDTO>> sendDoctorInfos(@RequestBody List<ABasDoctorInfoVO> list){
        return aBasDoctorInfoFacade.executeDoctor(list);
    }

    @ApiOperation(value = "数据引擎-获取医院所有有效科室的基本信息")
    @PostMapping("/sendDeptInfos")
    @SysLogger("sendDeptInfos")
    public RespDTO<List<ABasDeptInfoDTO>> sendDeptInfos(@RequestBody List<ABasDeptInfoVO> list){
        return aBasDeptInfoFacade.executeDept(list);
    }

    @ApiOperation(value = "数据引擎-获取医院所有文书模板基本信息")
    @PostMapping("/sendRecordTypes")
    @SysLogger("sendRecordTypes")
    public RespDTO<List<AMedicalRecordTypeDTO>> sendRecordTypes(@RequestBody List<AMedicalRecordTypeVO> list){
        return aMedRecordTypeFacade.executeMRType(list);
    }

    @ApiOperation(value = "数据引擎-获取医院病人住院记录信息")
    @PostMapping("/sendPatientInfo")
    @SysLogger("sendPatientInfo")
    public RespDTO<List<ABehospitalInfoDTO>> sendPatientInfo(@RequestBody List<ABehospitalInfoVO> list){
        return aBehospitalInfoFacade.executeBehospital(list);
    }

    @ApiOperation(value = "数据引擎-获取医院医生医嘱信息")
    @PostMapping("/sendDoctorAdvice")
    @SysLogger("sendDoctorAdvice")
    public RespDTO<List<ADoctorAdviceDTO>> sendDoctorAdvice(@RequestBody List<ADoctorAdviceVO> list){
        return aDoctorAdviceFacade.executeDoctorAdvice(list);
    }

    @ApiOperation(value = "支持评分-获取病历记录")
    @PostMapping("/sendMedicalRecord")
    @SysLogger("sendMedicalRecord")
    public RespDTO<Map<String, Object>> sendMedicalRecord(@Valid @RequestBody AMrContentVO aMrContentVO){
        return aMedicalRecordFacade.executeMrRecord(aMrContentVO);
    }

    @ApiOperation(value = "支持数据同步-获取病历记录")
    @PostMapping("/sendMrContent")
    @SysLogger("sendMrContent")
    public RespDTO<List<AMedicalRecordDTO>> sendMrContent(@RequestBody List<AMedicalRecordVO> list){
        return aMedicalRecordFacade.executeMrRecordView(list);
    }

    @ApiOperation(value = "数据引擎-获取病案首页")
    @PostMapping("/sendHomePage")
    @SysLogger("sendHomePage")
    public RespDTO<List<AHomePageDTO>> sendHomePage(@RequestBody List<AHomePageVO> list){
        return aHomePageFacade.executeHomePage(list);
    }

    @ApiOperation(value = "数据引擎-获取病案首页诊断")
    @PostMapping("/sendHomeDiagnose")
    @SysLogger("sendHomeDiagnose")
    @Ignore
    public RespDTO<List<AHomeDiagnoseDTO>> sendHomeDiagnose(@RequestBody List<AHomeDiagnoseVO> list){
        return aHomeDiagnoseInfoFacade.executeHomeDiagnose(list);
    }

    @ApiOperation(value = "数据引擎-获取病案首页手术")
    @PostMapping("/sendHomeOperation")
    @SysLogger("sendHomeOperation")
    @Ignore
    public RespDTO<List<AHomeOperationDTO>> sendHomeOperation(@RequestBody List<AHomeOperationVO> list){
        return RespDTO.onSuc(aHomeOperationInfoFacade.executeHomeOperation(list));
    }

    @ApiOperation(value = "数据引擎-获取危急值")
    @PostMapping("/sendCrisis")
    @SysLogger("sendCrisis")
    public RespDTO<List<AMedCrisisDTO>> sendCrisis(@RequestBody List<AMedCrisisVO> list){
        return aMedCrisisFacade.executeMedCrisis(list);
    }

}
